/* procmime.c */

#include <glib.h>
#include <stdio.h>
#include <string.h>

#include "main.h"
#include "procmime.h"
#include "procheader.h"
#include "utils.h"

MimeInfo *procmime_mimeinfo_new(void)
{
	MimeInfo *mimeinfo;

	mimeinfo = g_new0(MimeInfo, 1);
	mimeinfo->mime_type     = MIME_UNKNOWN;
	mimeinfo->encoding_type = ENC_UNKNOWN;

	return mimeinfo;
}

void procmime_mimeinfo_free(MimeInfo *mimeinfo)
{
	if (!mimeinfo) return;

	g_free(mimeinfo->encoding);
	g_free(mimeinfo->content_type);
	g_free(mimeinfo->charset);
	g_free(mimeinfo->name);
	g_free(mimeinfo->boundary);
	g_free(mimeinfo->content_disposition);
	g_free(mimeinfo);
}

void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
{
	gchar *buf;

	Xalloca(buf, strlen(encoding) + 1, return);
	strcpy(buf, encoding);

	g_free(mimeinfo->encoding);

	mimeinfo->encoding = g_strdup(g_strstrip(buf));
	if (!strcasecmp(buf, "7bit"))
		mimeinfo->encoding_type = ENC_7BIT;
	else if (!strcasecmp(buf, "quoted-printable"))
		mimeinfo->encoding_type = ENC_QUOTED_PRINTABLE;
	else if (!strcasecmp(buf, "base64"))
		mimeinfo->encoding_type = ENC_BASE64;
	else
		mimeinfo->encoding_type = ENC_UNKNOWN;

}

void procmime_scan_content_type(MimeInfo *mimeinfo, const gchar *content_type)
{
	gchar *delim, *p, *cnttype;
	gchar *buf;

	Xalloca(buf, strlen(content_type) + 1, return);
	strcpy(buf, content_type);

	g_free(mimeinfo->content_type);
	g_free(mimeinfo->charset);
	g_free(mimeinfo->name);
	mimeinfo->content_type = NULL;
	mimeinfo->charset      = NULL;
	mimeinfo->name         = NULL;

	if ((delim = strchr(buf, ';'))) *delim = '\0';
	mimeinfo->content_type = cnttype = g_strdup(g_strstrip(buf));

	if (!strncasecmp(cnttype, "text/", 5) ||
	    !strncasecmp(cnttype, "message/rfc822", 14))
		mimeinfo->mime_type = MIME_TEXT;
	else if (!strncasecmp(cnttype, "application/", 12))
		mimeinfo->mime_type = MIME_APPLICATION;
	else if (!strncasecmp(cnttype, "multipart/", 10))
		mimeinfo->mime_type = MIME_MULTIPART;
	else
		mimeinfo->mime_type = MIME_UNKNOWN;

	if (!delim) return;
	p = delim + 1;

	for (;;) {
		gchar *eq;

		if ((delim = strchr(p, ';'))) *delim = '\0';

		if ((eq = strchr(p, '='))) {
			gchar *attr, *value;

			*eq = '\0';
			attr = p;
			g_strstrip(attr);
			value = eq + 1;
			g_strstrip(value);
			if (*value == '"')
				extract_quote(value, '"');
			else {
				eliminate_parenthesis(value, '(', ')');
				g_strstrip(value);
			}

			if (*value) {
				if (!strcasecmp(attr, "charset"))
					mimeinfo->charset = g_strdup(value);
				else if (!strcasecmp(attr, "name"))
					mimeinfo->name = g_strdup(value);
				else if (!strcasecmp(attr, "boundary"))
					mimeinfo->boundary = g_strdup(value);
			}
		} else
			break;

		if (!delim) break;
		p = delim + 1;
	}
}

enum
{
	H_CONTENT_TRANSFER_ENCODING = 0,
	H_CONTENT_TYPE		    = 1,
	H_CONTENT_DISPOSITION	    = 2
};

MimeInfo *procmime_scan_mime_header(FILE *fp)
{
	static HeaderEntry hentry[] = {{"Content-Transfer-Encoding:",
							  NULL, FALSE},
				       {"Content-Type:", NULL, TRUE},
				       {"Content-Disposition:",
							  NULL, TRUE},
				       {NULL,		  NULL, FALSE}};
	gchar buf[BUFFSIZE];
	gint hnum;
	HeaderEntry *hp;
	MimeInfo *mimeinfo = NULL;

	g_return_val_if_fail(fp != NULL, NULL);

	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
	       != -1) {
		hp = hentry + hnum;

		if (H_CONTENT_TRANSFER_ENCODING == hnum) {
			if (!mimeinfo) mimeinfo = procmime_mimeinfo_new();
			procmime_scan_encoding
				(mimeinfo, buf + strlen(hp->name));
		} else if (H_CONTENT_TYPE == hnum) {
			if (!mimeinfo) mimeinfo = procmime_mimeinfo_new();
			procmime_scan_content_type
				(mimeinfo, buf + strlen(hp->name));
		} else if (H_CONTENT_DISPOSITION == hnum) {
			if (!mimeinfo) mimeinfo = procmime_mimeinfo_new();
			mimeinfo->content_disposition =
				g_strdup(g_strstrip(buf));
		}
	}

	return mimeinfo;
}
